@@ -12,7 +12,7 @@ setClass("OnDiskNamedSequences")  # VIRTUAL class with no slots
 ###   - seqlengthsFilepath()
 ###   - seqinfo API (implementing seqinfo() is enough to make seqlengths(),
 ###                  seqlevels(), etc... work)
-###   - [[ -- load full sequence as XString object
+###   - getListElement() -- load full sequence as XString derivative
 ### Default methods are provided for the following:
 ###   - length()
 ###   - seqnames()
@@ -30,18 +30,28 @@ setGeneric("loadSubseqsFromLinearSequence",
 
 ### Low-level utilities
 
-### Works on an XString object or any object 'x' for which seqlengths() is
-### defined.
+.check_getListElement_index <- function(i, what)
+{
+    if (!is.character(i))
+        stop(wmsg(what, " can only be subsetted by name"))
+    if (length(i) < 1L)
+        stop(wmsg("attempt to select less than one element"))
+    if (length(i) > 1L)
+        stop(wmsg("attempt to select more than one element"))
+}
+
+### Works on an XString derivative or any object 'x' for which seqlengths()
+### is defined.
 .get_seqlength <- function(x, seqname)
 {
     if (!isSingleString(seqname))
-        stop("'seqname' must be a single string")
+        stop(wmsg("'seqname' must be a single string"))
     if (is(x, "XString"))
         return(length(x))
     x_seqlengths <- seqlengths(x)
     idx <- match(seqname, names(x_seqlengths))
     if (is.na(idx))
-        stop("invalid sequence name: ", seqname)
+        stop(wmsg("invalid sequence name: ", seqname))
     x_seqlengths[[idx]]
 }
 
@@ -60,7 +70,7 @@ setMethod("show", "OnDiskNamedSequences",
     }
 )
 
-### Load regions from a single sequence as an XStringSet object.
+### Load regions from a single sequence as an XStringSet derivative.
 
 ### 'seqname' is ignored.
 setMethod("loadSubseqsFromLinearSequence", "XString",
@@ -70,15 +80,104 @@ setMethod("loadSubseqsFromLinearSequence", "XString",
 
 setMethod("loadSubseqsFromLinearSequence", "OnDiskNamedSequences",
     function(x, seqname, ranges)
-        loadSubseqsFromLinearSequence(x[[seqname]], seqname, ranges)
+    {
+        seq <- getListElement(x, seqname)
+        loadSubseqsFromLinearSequence(seq, seqname, ranges)
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### RdsNamedSequences objects
+###
+### The 'dirpath' slot should point to a directory that contains one .rds
+### file per XString derivative + a seqlengths.rds file that contains a
+### serialized named integer vector with the sequence names and lengths.
+###
+
+setClass("RdsNamedSequences",
+    contains=c("RdsCollection", "OnDiskNamedSequences"),
+    prototype=prototype(
+        elementType="XString"
+    )
+)
+
+setMethod("seqlevels", "RdsNamedSequences", function(x) names(x))
+
+setMethod("seqlengthsFilepath", "RdsNamedSequences",
+    function(x) file.path(path(x), "seqlengths.rds")
+)
+
+.read_seqlengths_from_file <- function(x) readRDS(seqlengthsFilepath(x))
+
+setMethod("seqlengths", "RdsNamedSequences",
+    function(x)
+    {
+        noext_ends <- nchar(x@filenames) - nchar(".rds")
+        noext_filenames <- substr(x@filenames, 1L, noext_ends)
+        setNames(.read_seqlengths_from_file(x)[noext_filenames], names(x))
+    }
+)
+
+setMethod("seqinfo", "RdsNamedSequences",
+    function(x)
+    {
+        x_seqlengths <- seqlengths(x)
+        Seqinfo(names(x_seqlengths), unname(x_seqlengths))
+    }
+)
+
+setAs("RdsCollection", "RdsNamedSequences",
+    function(from)
+    {
+        ans <- new2("RdsNamedSequences", from, elementType="XString",
+                                         check=FALSE)
+        seqlengths_from_file <- .read_seqlengths_from_file(ans)
+        if (!is.integer(seqlengths_from_file))
+            stop(wmsg("object serialized in ",
+                      "file '", seqlengthsFilepath(ans), "' ",
+                      "must be a named integer vector"))
+        if (!all(names(from) %in% names(seqlengths_from_file)))
+            stop(wmsg("the names on the RdsCollection object to coerce ",
+                      "to RdsNamedSequences must be a subset of ",
+                      "the names on the integer vector serialized ",
+                      "in file '", seqlengthsFilepath(ans), "'"))
+        ans
+    }
+)
+
+### Constructor.
+RdsNamedSequences <- function(path, seqnames)
+{
+    filenames <- paste0(seqnames, ".rds")
+    as(RdsCollection(path, filenames), "RdsNamedSequences")
+}
+
+### Load a full sequence as an XString derivative.
+setMethod("getListElement", "RdsNamedSequences",
+    function(x, i, exact=TRUE)
+    {
+        .check_getListElement_index(i, "an RdsNamedSequences object")
+        ans <- callNextMethod()
+        if (!is(ans, "XString")) {
+            filepath <- file.path(path(x), x@filenames[[i]])
+            stop(wmsg("serialized object in file '", filepath, "' ",
+                      "must be an XString derivative"))
+        }
+        updateObject(ans)
+    }
 )
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### RdaNamedSequences objects
 ###
-### The "dirpath" slot should contain 1 serialized XString object per
-### sequence + a serialized named integer vector ('seqlengths.rda')
+### June 2020: THE RdaNamedSequences CLASS IS SUPERSEDED BY THE
+### RdsNamedSequences CLASS!
+### TODO: Deprecate the RdaNamedSequences class.
+###
+### The "dirpath" slot should contain 1 serialized XString derivative
+### per sequence + a serialized named integer vector ('seqlengths.rda')
 ### containing the sequence names and lengths.
 ###
 
@@ -100,8 +199,8 @@ setMethod("seqlengths", "RdaNamedSequences",
     {
         ans <- x@seqlengths[["seqlengths"]]
         if (!is.integer(ans) || is.null(names(ans)))
-            stop("serialized object in file '", seqlengthsFilepath(x), "' ",
-                 "must be a named integer vector")
+            stop(wmsg("serialized object in file '", seqlengthsFilepath(x),
+                      "' must be a named integer vector"))
         ans
     }
 )
@@ -123,14 +222,15 @@ RdaNamedSequences <- function(dirpath, seqnames)
     new("RdaNamedSequences", sequences, seqlengths=seqlengths)
 }
 
-### Load a full sequence as an XString object.
-setMethod("[[", "RdaNamedSequences",
-    function(x, i, j, ...)
+### Load a full sequence as an XString derivative.
+setMethod("getListElement", "RdaNamedSequences",
+    function(x, i, exact=TRUE)
     {
-        ans <- callNextMethod()
+        .check_getListElement_index(i, "an RdaNamedSequences object")
+        ans <- x[[i]]
         if (!is(ans, "XString"))
-            stop("serialized object in file '", rdaPath(x, i), "' ",
-                 "must be an XString object")
+            stop(wmsg("serialized object in file '", rdaPath(x, i), "' ",
+                      "must be an XString derivative"))
         updateObject(ans)
     }
 )
@@ -165,19 +265,11 @@ FastaNamedSequences <- function(filepath)
     new("FastaNamedSequences", fafile=fafile)
 }
 
-### We only support subetting by name.
 ### Load a full sequence as a DNAString object.
-setMethod("[[", "FastaNamedSequences",
-    function(x, i, j, ...)
+setMethod("getListElement", "FastaNamedSequences",
+    function(x, i, exact=TRUE)
     {
-        if (!missing(j) || length(list(...)) > 0L)
-            stop("invalid subsetting")
-        if (!is.character(i))
-            stop("a FastaNamedSequences object can only be subsetted by name")
-        if (length(i) < 1L)
-            stop("attempt to select less than one element")
-        if (length(i) > 1L)
-            stop("attempt to select more than one element")
+        .check_getListElement_index(i, "a FastaNamedSequences object")
         fafile <- x@fafile
         seqlength <- .get_seqlength(fafile, i)
         param <- GRanges(i, IRanges(1L, seqlength))
@@ -193,13 +285,13 @@ setMethod("loadSubseqsFromLinearSequence", "FastaNamedSequences",
     function(x, seqname, ranges)
     {
         if (!is(ranges, "IntegerRanges"))
-            stop("'ranges' must be an IntegerRanges object")
+            stop(wmsg("'ranges' must be an IntegerRanges object"))
         fafile <- x@fafile
         seqlength <- .get_seqlength(fafile, seqname)
         if (length(ranges) != 0L &&
             (min(start(ranges)) < 1L || max(end(ranges)) > seqlength))
-            stop("trying to load regions beyond the boundaries ",
-                 "of non-circular sequence \"", seqname, "\"")
+            stop(wmsg("trying to load regions beyond the boundaries ",
+                      "of non-circular sequence \"", seqname, "\""))
         param <- GRanges(seqname, ranges)
         scanFa(fafile, param=param)
     }
@@ -234,19 +326,11 @@ TwobitNamedSequences <- function(filepath)
     new("TwobitNamedSequences", twobitfile=twobitfile)
 }
 
-### We only support subetting by name.
 ### Load a full sequence as a DNAString object.
-setMethod("[[", "TwobitNamedSequences",
-    function(x, i, j, ...)
+setMethod("getListElement", "TwobitNamedSequences",
+    function(x, i, exact=TRUE)
     {
-        if (!missing(j) || length(list(...)) > 0L)
-            stop("invalid subsetting")
-        if (!is.character(i))
-            stop("a FastaNamedSequences object can only be subsetted by name")
-        if (length(i) < 1L)
-            stop("attempt to select less than one element")
-        if (length(i) > 1L)
-            stop("attempt to select more than one element")
+        .check_getListElement_index(i, "a TwobitNamedSequences object")
         twobitfile <- x@twobitfile
         seqlength <- .get_seqlength(twobitfile, i)
         which <- GRanges(i, IRanges(1L, seqlength))
@@ -259,13 +343,13 @@ setMethod("loadSubseqsFromLinearSequence", "TwobitNamedSequences",
     function(x, seqname, ranges)
     {
         if (!is(ranges, "IntegerRanges"))
-            stop("'ranges' must be an IntegerRanges object")
+            stop(wmsg("'ranges' must be an IntegerRanges object"))
         twobitfile <- x@twobitfile
         seqlength <- .get_seqlength(twobitfile, seqname)
         if (length(ranges) != 0L &&
             (min(start(ranges)) < 1L || max(end(ranges)) > seqlength))
-            stop("trying to load regions beyond the boundaries ",
-                 "of non-circular sequence \"", seqname, "\"")
+            stop(wmsg("trying to load regions beyond the boundaries ",
+                      "of non-circular sequence \"", seqname, "\""))
         which <- GRanges(seqname, ranges)
         import(twobitfile, which=which)
     }
@@ -279,7 +363,7 @@ setMethod("loadSubseqsFromLinearSequence", "TwobitNamedSequences",
 .loadSubseqsFromCircularSequence <- function(x, seqname, ranges)
 {
     if (!is(ranges, "IntegerRanges"))
-        stop("'ranges' must be an IntegerRanges object")
+        stop(wmsg("'ranges' must be an IntegerRanges object"))
     seqlength <- .get_seqlength(x, seqname)
     LRranges <- splitLRranges(ranges, seqlength, seqname)
     Lans <- loadSubseqsFromLinearSequence(x, seqname, LRranges$L)
@@ -291,9 +375,9 @@ loadSubseqsFromStrandedSequence <- function(x, seqname, ranges, strand,
                                             is_circular=NA)
 {
     if (length(ranges) != length(strand))
-        stop("'ranges' and 'strand' must have the same length")
+        stop(wmsg("'ranges' and 'strand' must have the same length"))
     if (!is.logical(is_circular) || length(is_circular) != 1L)
-        stop("'is_circular' must be a single logical")
+        stop(wmsg("'is_circular' must be a single logical"))
     if (is_circular %in% c(NA, FALSE)) {
         loadFUN <- loadSubseqsFromLinearSequence
     } else {
@@ -301,7 +385,43 @@ loadSubseqsFromStrandedSequence <- function(x, seqname, ranges, strand,
     }
     ans <- loadFUN(x, seqname, ranges)
     idx <- which(strand == "-")
-    ans[idx] <- reverseComplement(ans[idx])
+    if (length(idx) != 0L)
+        ans[idx] <- reverseComplement(ans[idx])
     ans
 }
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### OnDiskNamedSequences() constructor
+###
+
+OnDiskNamedSequences <- function(dirpath, seqnames=NULL)
+{
+    filename <- "seqlengths.rds"
+    filepath <- file.path(dirpath, filename)
+    if (file.exists(filepath))
+        return(RdsNamedSequences(dirpath, seqnames))
+
+    filename <- "seqlengths.rda"
+    filepath <- file.path(dirpath, filename)
+    if (file.exists(filepath))
+        return(RdaNamedSequences(dirpath, seqnames))
+
+    filename <- "single_sequences.fa"
+    filepath <- file.path(dirpath, filename)
+    if (file.exists(filepath))
+        return(FastaNamedSequences(filepath))
+
+    filename <- "single_sequences.fa.rz"
+    filepath <- file.path(dirpath, filename)
+    if (file.exists(filepath))
+        return(FastaNamedSequences(filepath))
+
+    filename <- "single_sequences.2bit"
+    filepath <- file.path(dirpath, filename)
+    if (file.exists(filepath))
+        return(TwobitNamedSequences(filepath))
+
+    stop(wmsg("invalid directory content at ", dirpath))
+}
+
